/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    SubscriptionRepository.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import java.util.HashMap;
import java.util.Set;

import de.fhhannover.inform.ifmap.communication.Connection;
import de.fhhannover.inform.ifmap.util.CryptoUtil;

public class SubscriptionRepository {

	private HashMap<Connection, HashMap<String, IdentifierData>> mSubscriptions = null;
	private static SubscriptionRepository mInstance;
	
	private SubscriptionRepository() {
		mSubscriptions = new HashMap<Connection, HashMap<String, IdentifierData>>();
	}
	
	public static SubscriptionRepository getInstance(){
		if(mInstance == null){
			mInstance = new SubscriptionRepository();
		}
		return mInstance;
	}
	
	public void addSubscription(Connection con, IdentifierData ident){
		String uuid = CryptoUtil.generateMD5BySize(ident.getRequestObject().toString(), 16);
		HashMap<String, IdentifierData> map = mSubscriptions.get(con);
		if(map == null){
			map = new HashMap<String, IdentifierData>();			
		}
		map.put(uuid, ident);
		mSubscriptions.put(con, map);
	}
	
	public void removeSubscriptions(Connection con, String[] keys){
		HashMap<String, IdentifierData> map = mSubscriptions.get(con);
		if(map != null){
			for(String s : keys) {
				System.err.println("Removing "+s+" from Subs.Repos");
				map.remove(s);		
			}
		}		
	}		
	
	public HashMap<String, IdentifierData> getSubscriptions(Connection con) {
		return mSubscriptions.get(con);
	}

	public boolean isAlreadySubscribed(Connection con, IdentifierData ident){
		String uuid = CryptoUtil.generateMD5BySize(ident.getRequestObject().toString(), 16);
		HashMap<String, IdentifierData> map = mSubscriptions.get(con);
		if(map != null){
			if(map.get(uuid) != null){				
				return true;
			}
		}
		return false;
	}
	
	public boolean isAlreadySubscribed(Connection con, String uuid){		
		HashMap<String, IdentifierData> map = mSubscriptions.get(con);
		if(map != null){
			if(map.get(uuid) != null){
				return true;
			}
		}
		return false;
	}
	
	public IdentifierData getIdentifierDataBySubscriptionName(Connection con, String uuid){
		HashMap<String, IdentifierData> map = mSubscriptions.get(con);
		if(map != null){
			return map.get(uuid);			
		}
		return null;
	}
	
	public String getSubscriptionNameByIdentifier(Connection con, IfmapDataType ident){
		String name = null;
		HashMap<String, IdentifierData> map = mSubscriptions.get(con);
		if(map != null){
			if(map.containsValue(ident)){
				Set<String> keys = map.keySet();
				for(String key : keys){
					if(map.get(key).equals(ident)){
						name = key;
					}
				}					
			}
		}		
		return name;
	}
}
