/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.view.dialog
 * File:    ConnectionDialog.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.view.dialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import de.fhhannover.inform.ifmap.communication.Connection;
import de.fhhannover.inform.ifmap.communication.ConnectionFactory;
import de.fhhannover.inform.ifmap.communication.ConnectionParameter;
import de.fhhannover.inform.ifmap.exception.ConnectionCreationException;

public class ConnectionDialog extends JFrame {

	private static final long serialVersionUID = 6756137933419928418L;
	private JPanel contentPane;
	private JTextField textFieldEndpoint;
	private JTextField textFieldKeystore;
	private JPasswordField passwordFieldKeystore;
	private JTextField textFieldTruststore;
	private JPasswordField passwordFieldTruststore;
	private JTextField textFieldName;
	private JTextField textFieldUsername;
	private JPasswordField textFieldPassword;
	private JLabel labelUsername = null;
	private JLabel labelPassword;
	private ConnectionParameter previousParam = null;
	private JCheckBox checkBoxBasicAuth;
	public JTextArea testPanel;
	public JButton buttonClose;
	public DefaultListModel mListModel;
	public JCheckBox checkBoxDump;
	public JList mConnectionList;
	private JPanel panelCenter;
	private JPanel panelParameter;
	private JPanel panelBlank;
	private int dupCounter = 0;
	public JButton buttonTest;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Throwable e) {
			e.printStackTrace();
		}
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					ConnectionDialog frame = new ConnectionDialog(null);

					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	public void assignParamsToObject() {
		ConnectionParameter param = (ConnectionParameter) mConnectionList
				.getSelectedValue();
		if (param != null) {
			param.setName(textFieldName.getText().trim());
			param.setEndpoint(textFieldEndpoint.getText().trim());
			param.setTruststore(textFieldTruststore.getText().trim());
			param.setTruststorePass(new String(passwordFieldTruststore
					.getPassword()));
			param.setKeystore(textFieldKeystore.getText().trim());
			param.setKeystorePass(new String(passwordFieldKeystore
					.getPassword()));
			param.setBasicAuthEnabled(checkBoxBasicAuth.isSelected());
			param.setBasicauthUser(textFieldUsername.getText().trim());
			param.setBasicauthPass(new String(textFieldPassword.getPassword()));
			param.setAutoSubscribe(checkBoxDump.isSelected());
		}
	}

	/**
	 * Create the frame.
	 */
	public ConnectionDialog(Connection[] cons) {
		setResizable(false);
		setTitle("Manage IF-MAP connections");
		setBounds(100, 100, 671, 628);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);

		JSplitPane splitPane = new JSplitPane();
		contentPane.add(splitPane, BorderLayout.CENTER);

		mConnectionList = new JList();
		// mConnectionList.setCellRenderer(new ConnectionListRenderer());
		mConnectionList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		mConnectionList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent arg0) {
				ConnectionParameter param = (ConnectionParameter) mConnectionList
						.getSelectedValue();
				if (param == null) {
					return;
				}
				if (previousParam != null) {
					previousParam.setName(textFieldName.getText().trim());
					previousParam.setEndpoint(textFieldEndpoint.getText()
							.trim());
					previousParam.setTruststore(textFieldTruststore.getText()
							.trim());
					previousParam.setTruststorePass(new String(
							passwordFieldTruststore.getPassword()));
					previousParam.setKeystore(textFieldKeystore.getText()
							.trim());
					previousParam.setKeystorePass(new String(
							passwordFieldKeystore.getPassword()));
					previousParam.setBasicAuthEnabled(checkBoxBasicAuth
							.isSelected());
					previousParam.setBasicauthUser(textFieldUsername.getText()
							.trim());
					previousParam.setBasicauthPass(new String(textFieldPassword
							.getPassword()));
					previousParam.setAutoSubscribe(checkBoxDump.isSelected());
				}

				// fill fields with actual parameters
				textFieldName.setText(param.getName());
				textFieldEndpoint.setText(param.getEndpoint());
				textFieldKeystore.setText(param.getKeystore());
				if (param.getKeystorePass() != null) {
					passwordFieldKeystore.setText(new String(param
							.getKeystorePass()));
				} else {
					passwordFieldKeystore.setText(null);
				}
				textFieldTruststore.setText(param.getTruststore());
				if (param.getTruststorePass() != null) {
					passwordFieldTruststore.setText(new String(param
							.getTruststorePass()));
				} else {
					passwordFieldTruststore.setText(null);
				}
				checkBoxBasicAuth.setSelected(param.isBasicAuthEnabled());
				textFieldUsername.setText(param.getBasicauthUser());
				if (param.getBasicauthPass() != null) {
					textFieldPassword.setText(new String(param
							.getBasicauthPass()));
				} else {
					textFieldPassword.setText(null);
				}
				checkBoxDump.setSelected(param.isAutoSubscribe());
				labelUsername.setEnabled(param.isBasicAuthEnabled());
				textFieldUsername.setEnabled(param.isBasicAuthEnabled());
				labelPassword.setEnabled(param.isBasicAuthEnabled());
				textFieldPassword.setEnabled(param.isBasicAuthEnabled());
				previousParam = param;
			}
		});
		mConnectionList.setBorder(new LineBorder(
				SystemColor.activeCaptionBorder));
		mListModel = new DefaultListModel();
		mConnectionList.setModel(mListModel);
		if (cons != null) {
			for (Connection c : cons) {
				mListModel.add(mListModel.getSize(),
						c.getConnectionParameters());
			}
		}

		splitPane.setLeftComponent(mConnectionList);

		panelCenter = new JPanel();
		panelCenter.setBorder(new LineBorder(SystemColor.activeCaptionBorder));
		splitPane.setRightComponent(panelCenter);
		panelCenter.setLayout(new BorderLayout(0, 0));

		panelParameter = new JPanel();

		panelBlank = new JPanel();
		panelCenter.add(panelBlank, BorderLayout.CENTER);
		panelBlank.setLayout(null);

		JLabel lblNoConnectionsYet = new JLabel("No connections yet.");
		lblNoConnectionsYet.setBounds(0, 234, 497, 16);
		lblNoConnectionsYet.setHorizontalAlignment(SwingConstants.CENTER);
		panelBlank.add(lblNoConnectionsYet);
		panelParameter.setLayout(null);
		// panelCenter.add(panelParameter, BorderLayout.CENTER);

		JLabel label = new JLabel("Endpoint");
		label.setHorizontalAlignment(SwingConstants.RIGHT);
		label.setBounds(3, 38, 130, 16);
		panelParameter.add(label);

		textFieldEndpoint = new JTextField();
		textFieldEndpoint.setColumns(10);
		textFieldEndpoint.setBounds(145, 38, 336, 22);
		panelParameter.add(textFieldEndpoint);

		JLabel label_1 = new JLabel("Truststore");
		label_1.setHorizontalAlignment(SwingConstants.RIGHT);
		label_1.setBounds(3, 126, 130, 16);
		panelParameter.add(label_1);

		JLabel label_2 = new JLabel("Truststore password");
		label_2.setHorizontalAlignment(SwingConstants.RIGHT);
		label_2.setBounds(3, 155, 130, 16);
		panelParameter.add(label_2);

		textFieldKeystore = new JTextField();
		textFieldKeystore.setColumns(10);
		textFieldKeystore.setBounds(145, 67, 227, 22);
		panelParameter.add(textFieldKeystore);

		JButton button = new JButton("Choose...");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser = new JFileChooser(new File("."));
				int returnVal = chooser.showOpenDialog(ConnectionDialog.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					textFieldKeystore.setText(chooser.getSelectedFile()
							.getAbsolutePath());
				}
			}
		});
		button.setBounds(384, 66, 97, 25);
		panelParameter.add(button);

		passwordFieldKeystore = new JPasswordField();
		passwordFieldKeystore.setBounds(145, 96, 227, 22);
		panelParameter.add(passwordFieldKeystore);

		JLabel label_3 = new JLabel("Keystore");
		label_3.setHorizontalAlignment(SwingConstants.RIGHT);
		label_3.setBounds(3, 67, 130, 16);
		panelParameter.add(label_3);

		JLabel label_4 = new JLabel("Keystore password");
		label_4.setHorizontalAlignment(SwingConstants.RIGHT);
		label_4.setBounds(3, 96, 130, 16);
		panelParameter.add(label_4);

		textFieldTruststore = new JTextField();
		textFieldTruststore.setColumns(10);
		textFieldTruststore.setBounds(145, 126, 227, 22);
		panelParameter.add(textFieldTruststore);

		passwordFieldTruststore = new JPasswordField();
		passwordFieldTruststore.setBounds(145, 155, 227, 22);
		panelParameter.add(passwordFieldTruststore);

		JButton button_1 = new JButton("Choose...");
		button_1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser = new JFileChooser(new File("."));
				int returnVal = chooser.showOpenDialog(ConnectionDialog.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					textFieldTruststore.setText(chooser.getSelectedFile()
							.getAbsolutePath());
				}
			}
		});
		button_1.setBounds(384, 125, 97, 25);
		panelParameter.add(button_1);

		JLabel label_5 = new JLabel("Name");
		label_5.setHorizontalAlignment(SwingConstants.RIGHT);
		label_5.setBounds(0, 9, 133, 16);
		panelParameter.add(label_5);

		textFieldName = new JTextField();
		textFieldName.setColumns(10);
		textFieldName.setBounds(145, 9, 227, 22);
		panelParameter.add(textFieldName);

		JLabel label_6 = new JLabel("Basic Authentication");
		label_6.setHorizontalAlignment(SwingConstants.RIGHT);
		label_6.setBounds(3, 184, 130, 16);
		panelParameter.add(label_6);

		checkBoxBasicAuth = new JCheckBox("");
		checkBoxBasicAuth.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				boolean selected = checkBoxBasicAuth.isSelected();
				labelUsername.setEnabled(selected);
				textFieldUsername.setEnabled(selected);
				labelPassword.setEnabled(selected);
				textFieldPassword.setEnabled(selected);
			}
		});
		checkBoxBasicAuth.setBounds(141, 180, 25, 25);
		panelParameter.add(checkBoxBasicAuth);

		labelUsername = new JLabel("Username");
		labelUsername.setHorizontalAlignment(SwingConstants.RIGHT);
		labelUsername.setEnabled(false);
		labelUsername.setBounds(3, 205, 130, 16);
		panelParameter.add(labelUsername);

		textFieldUsername = new JTextField();
		textFieldUsername.setEnabled(false);
		textFieldUsername.setColumns(10);
		textFieldUsername.setBounds(145, 205, 227, 22);
		panelParameter.add(textFieldUsername);

		labelPassword = new JLabel("Password");
		labelPassword.setHorizontalAlignment(SwingConstants.RIGHT);
		labelPassword.setEnabled(false);
		labelPassword.setBounds(3, 237, 130, 16);
		panelParameter.add(labelPassword);

		textFieldPassword = new JPasswordField();
		textFieldPassword.setEnabled(false);
		textFieldPassword.setBounds(145, 237, 227, 22);
		panelParameter.add(textFieldPassword);

		buttonTest = new JButton("Test connection");
		buttonTest.setBounds(12, 332, 123, 25);
		panelParameter.add(buttonTest);

		JButton btnClear = new JButton("Clear");
		btnClear.setBounds(69, 370, 64, 25);
		panelParameter.add(btnClear);

		testPanel = new JTextArea();
		testPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		testPanel.setBounds(145, 332, 336, 166);
		panelParameter.add(testPanel);
		testPanel.setLayout(null);

		JLabel lblDump = new JLabel("Dump");
		lblDump.setHorizontalAlignment(SwingConstants.RIGHT);
		lblDump.setBounds(36, 277, 97, 16);
		panelParameter.add(lblDump);

		checkBoxDump = new JCheckBox("");
		checkBoxDump.setBounds(145, 268, 25, 25);
		panelParameter.add(checkBoxDump);
		checkBoxDump.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (checkBoxDump.isSelected()) {
					JOptionPane
							.showMessageDialog(
									contentPane,
									"Dumping is NOT IF-MAP 2.0 compliant and can only be used with irond.",
									"Warning", JOptionPane.WARNING_MESSAGE);
				}
			}
		});

		btnClear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				textFieldEndpoint.setText("");
				textFieldKeystore.setText("");
				passwordFieldKeystore.setText("");
				textFieldTruststore.setText("");
				passwordFieldTruststore.setText("");
				checkBoxBasicAuth.setSelected(false);
				textFieldUsername.setText("");
				textFieldPassword.setText("");
				textFieldUsername.setEnabled(false);
				textFieldPassword.setEnabled(false);
				labelUsername.setEnabled(false);
				labelPassword.setEnabled(false);
				checkBoxDump.setSelected(false);
			}
		});
		splitPane.setDividerLocation(150);

		JPanel panel_1 = new JPanel();
		contentPane.add(panel_1, BorderLayout.NORTH);
		panel_1.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));

		JButton addButton = new JButton("New");
		addButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				String name = "New Connection (" + (mListModel.getSize() + 1)
						+ ")";
				ConnectionParameter param = null;
				try {
					param = ConnectionFactory.createConnectionParameter(name);
				} catch (ConnectionCreationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				mListModel.add(mListModel.getSize(), param);
				panelCenter.remove(panelBlank);
				panelCenter.add(panelParameter, BorderLayout.CENTER);
				panelCenter.updateUI();
				mConnectionList.setSelectedIndex(mListModel.getSize() - 1);

			}
		});
		panel_1.add(addButton);

		JButton removeButton = new JButton("Delete");
		removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int index = mConnectionList.getSelectedIndex();
				if (index >= 0) {
					mListModel.remove(index);
					if (!mListModel.isEmpty()) {
						index = (index == 0) ? 0 : index - 1;
						mConnectionList.setSelectedIndex(index);
					} else {
						panelCenter.remove(panelParameter);
						panelCenter.add(panelBlank, BorderLayout.CENTER);
						panelCenter.updateUI();
					}
				}
			}
		});
		panel_1.add(removeButton);

		JButton copyButton = new JButton("Duplicate");
		copyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int index = mConnectionList.getSelectedIndex();
				if (index >= 0) {
					ConnectionParameter param = ((ConnectionParameter) mListModel
							.getElementAt(index)).clone();
					String name = null;
					int id = param.getName().lastIndexOf('(');
					if (id > -1) {
						name = param.getName().substring(0, id - 1) + " ("
								+ (++dupCounter) + ")";
					} else {
						name = param.getName() + " (" + (++dupCounter) + ")";
					}
					param.setName(name);
					mListModel.add(mListModel.getSize(), param);
					panelCenter.remove(panelBlank);
					panelCenter.add(panelParameter, BorderLayout.CENTER);
					panelCenter.updateUI();
					mConnectionList.setSelectedIndex(mListModel.getSize() - 1);
				}
			}
		});
		panel_1.add(copyButton);

		JPanel panel_2 = new JPanel();
		FlowLayout flowLayout = (FlowLayout) panel_2.getLayout();
		flowLayout.setAlignment(FlowLayout.RIGHT);
		contentPane.add(panel_2, BorderLayout.SOUTH);

		buttonClose = new JButton("Close");
		/*
		 * buttonClose.addActionListener(new ActionListener() { public void
		 * actionPerformed(ActionEvent arg0) { int length =
		 * mConnectionList.getModel().getSize(); if (length > 0) {
		 * MyFrame.mListModel.clear(); Connection[] connections = new
		 * Connection[length]; for (int i = 0; i < length; i++) {
		 * ConnectionParameter param = (ConnectionParameter) mConnectionList
		 * .getModel().getElementAt(i); try { Connection con = ConnectionFactory
		 * .createConnection(param); connections[i] = con;
		 * MyFrame.mListModel.addElement(param); } catch
		 * (ConnectionCreationException e) { // TODO Auto-generated catch block
		 * e.printStackTrace(); } }
		 * //System.err.println("length:"+connections.length);
		 * MyFrame.mIfmapFacade.addConnections(connections); }
		 * setVisible(false); } });
		 */
		panel_2.add(buttonClose);

		if (!mListModel.isEmpty()) {
			mConnectionList.setSelectedIndex(0);
		}
	}

	public Connection[] getConnections() {
		Connection[] connections = null;
		int length = mListModel.getSize();
		if (length > 0) {
			connections = new Connection[length];
			for (int i = 0; i < length; i++) {
				ConnectionParameter param = (ConnectionParameter) mListModel
						.getElementAt(i);
				try {
					Connection con = ConnectionFactory.createConnection(param);
					connections[i] = con;
				} catch (ConnectionCreationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		return connections;
	}

	public void addConnections(Connection[] cons) {
		if (cons != null && cons.length > 0) {
			mListModel.clear();
			for (int i = 0; i < cons.length; i++) {
				mListModel.add(i, cons[i].getConnectionParameters());
			}
			panelCenter.remove(panelBlank);
			panelCenter.add(panelParameter, BorderLayout.CENTER);
			panelCenter.updateUI();
			if (!mListModel.isEmpty()) {
				mConnectionList.setSelectedIndex(0);
			}
		}
	}
}
